@@ -1,7 +1,7 @@
 /*
     Qalculate (library)
 
-    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna_k@fmgirl.com)
+    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -356,19 +356,20 @@ bool ExpressionItem::isLocal() const {
 	return b_local;
 }
 bool ExpressionItem::setLocal(bool is_local, int will_be_active) {
+	if(b_builtin) return false;
 	if(is_local != b_local) {
 		if(!b_local) {
 			bool was_active = b_active;
-			b_active = false;	
+			b_active = false;
 			ExpressionItem *item = copy();
-			b_local = is_local;	
+			b_local = is_local;
 			b_active = was_active;
 			if(will_be_active) {
 				setActive(true);
 			} else if(will_be_active == 0) {
 				setActive(false);
 			}
-			CALCULATOR->addExpressionItem(item);		
+			CALCULATOR->addExpressionItem(item);
 			if(was_active != item->isActive()) {
 				item->setChanged(true);
 			}
@@ -407,8 +408,8 @@ int ExpressionItem::precision() const {
 void ExpressionItem::setPrecision(int prec) {
 	if(i_precision != prec) {
 		i_precision = prec;
-		if(i_precision > 0) b_approx = true;
-		b_changed = true;	
+		if(i_precision >= 0) b_approx = true;
+		b_changed = true;
 	}
 }
 bool ExpressionItem::isActive() const {
