@@ -1,7 +1,7 @@
 /*
     Qalculate (library)
 
-    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna_k@fmgirl.com)
+    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@ bool Assumptions::isReal() {return i_type >= ASSUMPTION_TYPE_REAL;}
 bool Assumptions::isComplex() {return i_type == ASSUMPTION_TYPE_COMPLEX;}
 bool Assumptions::isNonZero() {return i_sign == ASSUMPTION_SIGN_NONZERO || isPositive() || isNegative();}
 bool Assumptions::isNonMatrix() {return i_type >= ASSUMPTION_TYPE_NONMATRIX;}
+bool Assumptions::isScalar() {return i_type >= ASSUMPTION_TYPE_NONMATRIX;}
 
 AssumptionType Assumptions::type() {return i_type;}
 AssumptionSign Assumptions::sign() {return i_sign;}
@@ -184,6 +185,10 @@ bool UnknownVariable::representsNonMatrix() {
 	if(o_assumption) return o_assumption->isNonMatrix();
 	return CALCULATOR->defaultAssumptions()->isNonMatrix();
 }
+bool UnknownVariable::representsScalar() { 
+	if(o_assumption) return o_assumption->isScalar();
+	return CALCULATOR->defaultAssumptions()->isScalar();
+}
 
 KnownVariable::KnownVariable(string cat_, string name_, const MathStructure &o, string title_, bool is_local, bool is_builtin, bool is_active) : Variable(cat_, name_, title_, is_local, is_builtin, is_active) {
 	mstruct = new MathStructure(o);
@@ -191,12 +196,20 @@ KnownVariable::KnownVariable(string cat_, string name_, const MathStructure &o,
 	setPrecision(mstruct->precision());
 	b_expression = false;
 	sexpression = "";
-	calculated_precision = 0;
+	suncertainty = "";
+	sunit = "";
+	calculated_precision = -1;
+	calculated_with_interval = false;
+	calculated_with_units = false;
 	setChanged(false);
 }
 KnownVariable::KnownVariable(string cat_, string name_, string expression_, string title_, bool is_local, bool is_builtin, bool is_active) : Variable(cat_, name_, title_, is_local, is_builtin, is_active) {
 	mstruct = NULL;
-	calculated_precision = 0;
+	calculated_precision = -1;
+	calculated_with_interval = false;
+	calculated_with_units = false;
+	suncertainty = "";
+	sunit = "";
 	set(expression_);
 	setChanged(false);
 }
@@ -219,10 +232,20 @@ bool KnownVariable::isExpression() const {
 string KnownVariable::expression() const {
 	return sexpression;
 }
+string KnownVariable::uncertainty() const {
+	return suncertainty;
+}
+string KnownVariable::unit() const {
+	return sunit;
+}
 void KnownVariable::set(const ExpressionItem *item) {
 	if(item->type() == TYPE_VARIABLE && item->subtype() == SUBTYPE_KNOWN_VARIABLE) {
-		calculated_precision = 0;
+		calculated_precision = -1;
+		calculated_with_interval = false;
+		calculated_with_units = false;
 		sexpression = ((KnownVariable*) item)->expression();
+		suncertainty = ((KnownVariable*) item)->uncertainty();
+		sunit = ((KnownVariable*) item)->unit();
 		b_expression = ((KnownVariable*) item)->isExpression();
 		if(!b_expression) {
 			set(((KnownVariable*) item)->get());
@@ -235,9 +258,12 @@ void KnownVariable::set(const MathStructure &o) {
 	else mstruct->set(o);
 	setApproximate(mstruct->isApproximate());
 	setPrecision(mstruct->precision());
-	calculated_precision = 0;
+	calculated_precision = -1;
+	calculated_with_interval = false;
+	calculated_with_units = false;
 	b_expression = false;
 	sexpression = "";
+	setApproximate(o.isApproximate());
 	setChanged(true);
 }
 void KnownVariable::set(string expression_) {
@@ -248,17 +274,44 @@ void KnownVariable::set(string expression_) {
 	b_expression = true;
 	sexpression = expression_;
 	remove_blank_ends(sexpression);
-	calculated_precision = 0;
+	calculated_precision = -1;
+	calculated_with_interval = false;
+	calculated_with_units = false;
+	setChanged(true);
+}
+void KnownVariable::setUncertainty(string standard_uncertainty) {
+	if(mstruct) {
+		delete mstruct;
+	}	
+	mstruct = NULL;
+	suncertainty = standard_uncertainty;
+	remove_blank_ends(suncertainty);
+	calculated_precision = -1;
+	calculated_with_interval = false;
+	calculated_with_units = false;
+	if(!suncertainty.empty()) setApproximate(true);
+	setChanged(true);
+}
+void KnownVariable::setUnit(string unit_expression) {
+	if(mstruct) {
+		delete mstruct;
+	}	
+	mstruct = NULL;
+	sunit = unit_expression;
+	remove_blank_ends(sunit);
+	calculated_precision = -1;
+	calculated_with_interval = false;
+	calculated_with_units = false;
 	setChanged(true);
 }
 bool set_precision_of_numbers(MathStructure &mstruct, int i_prec) {
 	if(mstruct.isNumber()) {
-		if(i_prec < 1) {
+		if(i_prec < 0) {
 			if(!mstruct.number().isApproximate()) {
 				mstruct.number().setApproximate();
 				mstruct.numberUpdated();
 			}
-		} else if(mstruct.number().precision() < 1 || mstruct.number().precision() < i_prec) {
+		} else if(mstruct.number().precision() < 0 || mstruct.number().precision() < i_prec) {
 			mstruct.number().setPrecision(i_prec);
 			mstruct.numberUpdated();
 		}
@@ -274,15 +327,29 @@ bool set_precision_of_numbers(MathStructure &mstruct, int i_prec) {
 	return b;
 }
 const MathStructure &KnownVariable::get() {
-	if(b_expression && !mstruct) {
-		ParseOptions po;
-		if(isApproximate() && precision() < 1) {
-			po.read_precision = READ_PRECISION_WHEN_DECIMALS;
-		}
+	if(b_expression && ((!mstruct || mstruct->isAborted()) || calculated_with_interval != CALCULATOR->usesIntervalArithmetic() || (!sunit.empty() && calculated_with_units != CALCULATOR->variableUnitsEnabled()))) {
+		if(mstruct) mstruct->unref();
 		mstruct = new MathStructure();
-		CALCULATOR->parse(mstruct, sexpression, po);
-		if(precision() > 0) {
-			if(mstruct->precision() < 1 || precision() < mstruct->precision()) {
+		ParseOptions po;
+		if(isApproximate() && precision() == -1 && suncertainty.empty()) {
+			po.read_precision = ALWAYS_READ_PRECISION;
+		}
+		if(sunit.empty()) {
+			mstruct->setAborted();
+			CALCULATOR->parse(mstruct, sexpression, po);
+		} else {
+			mstruct->number().set(sexpression, po);
+			mstruct->numberUpdated();
+		}
+		if(!suncertainty.empty()) {
+			Number nr_u(suncertainty);
+			if(mstruct->isNumber()) {
+				mstruct->number().setUncertainty(nr_u);
+			} else if(mstruct->isMultiplication() && mstruct->size() > 0 && (*mstruct)[0].isNumber()) {
+				(*mstruct)[0].number().setUncertainty(nr_u);
+			}
+		} else if(precision() >= 0) {
+			if(mstruct->precision() < 0 || precision() < mstruct->precision()) {
 				if(!set_precision_of_numbers(*mstruct, precision())) mstruct->setPrecision(precision(), true);
 			}
 		} else if(isApproximate()) {
@@ -290,6 +357,18 @@ const MathStructure &KnownVariable::get() {
 				if(!set_precision_of_numbers(*mstruct, precision())) mstruct->setApproximate(true, true);
 			}
 		}
+		if(!sunit.empty() && CALCULATOR->variableUnitsEnabled()) {
+			MathStructure *mstruct_unit = new MathStructure;
+			mstruct_unit->setAborted();
+			CALCULATOR->parse(mstruct_unit, sunit, po);
+			mstruct->multiply_nocopy(mstruct_unit);
+		}
+		calculated_with_interval = CALCULATOR->usesIntervalArithmetic();
+		calculated_with_units = CALCULATOR->variableUnitsEnabled();
+	}
+	if(mstruct->contains(this, false, true, true) > 0) {
+		CALCULATOR->error(true, _("Recursive variable: %s = %s"), name().c_str(), mstruct->print().c_str(), NULL);
+		return m_undefined;
 	}
 	return *mstruct;
 }
@@ -298,6 +377,8 @@ bool KnownVariable::representsNegative(bool allow_units) {return get().represent
 bool KnownVariable::representsNonNegative(bool allow_units) {return get().representsNonNegative(allow_units);}
 bool KnownVariable::representsNonPositive(bool allow_units) {return get().representsNonPositive(allow_units);}
 bool KnownVariable::representsInteger(bool allow_units) {return get().representsInteger(allow_units);}
+bool KnownVariable::representsNonInteger(bool allow_units) {return get().representsNonInteger(allow_units);}
+bool KnownVariable::representsFraction(bool allow_units) {return get().representsFraction(allow_units);}
 bool KnownVariable::representsNumber(bool allow_units) {return get().representsNumber(allow_units);}
 bool KnownVariable::representsRational(bool allow_units) {return get().representsRational(allow_units);}
 bool KnownVariable::representsReal(bool allow_units) {return get().representsReal(allow_units);}
@@ -308,10 +389,13 @@ bool KnownVariable::representsOdd(bool allow_units) {return get().representsOdd(
 bool KnownVariable::representsUndefined(bool include_childs, bool include_infinite, bool be_strict) {return get().representsUndefined(include_childs, include_infinite, be_strict);}
 bool KnownVariable::representsBoolean() {return get().representsBoolean();}
 bool KnownVariable::representsNonMatrix() {return get().representsNonMatrix();}
+bool KnownVariable::representsScalar() {return get().representsScalar();}
 
 DynamicVariable::DynamicVariable(string cat_, string name_, string title_, bool is_local, bool is_builtin, bool is_active) : KnownVariable(cat_, name_, MathStructure(), title_, is_local, is_builtin, is_active) {
 	mstruct = NULL;
-	calculated_precision = 0;
+	calculated_precision = -1;
+	calculated_with_interval = false;
+	calculated_with_units = false;
 	setApproximate();
 	setChanged(false);
 }
@@ -323,8 +407,10 @@ DynamicVariable::DynamicVariable(const DynamicVariable *variable) {
 }
 DynamicVariable::DynamicVariable() : KnownVariable() {
 	mstruct = NULL;
-	calculated_precision = 0;
-	setApproximate();	
+	calculated_precision = -1;
+	calculated_with_interval = false;
+	calculated_with_units = false;
+	setApproximate();
 	setChanged(false);
 }
 DynamicVariable::~DynamicVariable() {
@@ -336,9 +422,12 @@ void DynamicVariable::set(const ExpressionItem *item) {
 void DynamicVariable::set(const MathStructure&) {}
 void DynamicVariable::set(string) {}
 const MathStructure &DynamicVariable::get() {
-	if(calculated_precision != CALCULATOR->getPrecision() || !mstruct) {
+	if(calculated_with_interval != CALCULATOR->usesIntervalArithmetic() || calculated_precision != CALCULATOR->getPrecision() || !mstruct || mstruct->isAborted()) {
+		if(mstruct) mstruct->unref();
 		mstruct = new MathStructure();
+		mstruct->setAborted();
 		calculated_precision = CALCULATOR->getPrecision();
+		calculated_with_interval = CALCULATOR->usesIntervalArithmetic();
 		calculate();
 	}
 	return *mstruct;
@@ -346,7 +435,9 @@ const MathStructure &DynamicVariable::get() {
 int DynamicVariable::calculatedPrecision() const {
 	return calculated_precision;
 }
-
+bool DynamicVariable::calculatedWithInterval() const {
+	return calculated_with_interval;
+}
 
 PiVariable::PiVariable() : DynamicVariable("Constants", "pi") {}
 void PiVariable::calculate() const {
@@ -364,4 +455,10 @@ CatalanVariable::CatalanVariable() : DynamicVariable("Constants", "catalan") {}
 void CatalanVariable::calculate() const {
 	Number nr; nr.catalan(); mstruct->set(nr);
 }
+PrecisionVariable::PrecisionVariable() : DynamicVariable("", "precision") {
+	setApproximate(false);
+}
+void PrecisionVariable::calculate() const {
+	mstruct->set(PRECISION, 1, 0);
+}
 
