@@ -1,7 +1,7 @@
 /*
     Qalculate
 
-    Copyright (C) 2004  Hanna Knutsson (hanna_k@fmgirl.com)
+    Copyright (C) 2004  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <vector>
-#include <glib.h>
 
 KnownVariable *vans[5];
 PrintOptions printops;
@@ -92,7 +91,7 @@ void generate_units_tree_struct() {
 					break;
 				}
 			}
-			if(!b) ia_units.push_back((void*) CALCULATOR->units[i]);						
+			if(!b) ia_units.push_back((void*) CALCULATOR->units[i]);
 		} else {
 			tree_struct *item = &unit_cats;
 			if(!CALCULATOR->units[i]->category().empty()) {
@@ -168,7 +167,7 @@ void generate_variables_tree_struct() {
 					break;
 				}
 			}
-			if(!b) ia_variables.push_back((void*) CALCULATOR->variables[i]);											
+			if(!b) ia_variables.push_back((void*) CALCULATOR->variables[i]);
 		} else {
 			tree_struct *item = &variable_cats;
 			if(!CALCULATOR->variables[i]->category().empty()) {
@@ -245,7 +244,7 @@ void generate_functions_tree_struct() {
 					break;
 				}
 			}
-			if(!b) ia_functions.push_back((void*) CALCULATOR->functions[i]);								
+			if(!b) ia_functions.push_back((void*) CALCULATOR->functions[i]);
 		} else {
 			tree_struct *item = &function_cats;
 			if(!CALCULATOR->functions[i]->category().empty()) {
@@ -357,6 +356,10 @@ void print_function(MathFunction *f) {
 		if(!f->description().empty()) {
 			fprintf(ffile, "<para>%s</para>\n", fix(f->description()).c_str());
 		}
+		if(!f->example(true).empty()) {
+			str = _("Example:"); str += " "; str += fix(f->example(false, ename->name));
+			fprintf(ffile, "<para>%s</para>\n", str.c_str());
+		}
 		if(f->subtype() == SUBTYPE_DATA_SET && !((DataSet*) f)->copyright().empty()) {
 			fprintf(ffile, "<para>%s</para>\n", fix(((DataSet*) f)->copyright()).c_str());
 		}
@@ -451,8 +454,18 @@ void print_variable(Variable *v) {
 		if(is_answer_variable(v)) {
 			value = _("a previous result");
 		} else if(v->isKnown()) {
-			if(((KnownVariable*) v)->isExpression()) {
+			if(v == CALCULATOR->v_precision) {
+				value = _("current precision");
+			} else if(((KnownVariable*) v)->isExpression()) {
 				value = fix(CALCULATOR->localizeExpression(((KnownVariable*) v)->expression()));
+				if(!((KnownVariable*) v)->uncertainty().empty()) {
+					value += "±";
+					value += fix(CALCULATOR->localizeExpression(((KnownVariable*) v)->uncertainty()));
+				}
+				if(!((KnownVariable*) v)->unit().empty()) {
+					value += " ";
+					value += fix(CALCULATOR->localizeExpression(((KnownVariable*) v)->unit()));
+				}
 				if(value.length() > 40) {
 					value = value.substr(0, 30);
 					value += "...";
@@ -463,7 +476,7 @@ void print_variable(Variable *v) {
 				} else if(((KnownVariable*) v)->get().isVector()) {
 					value = _("vector");
 				} else {
-					value = fix(CALCULATOR->printMathStructureTimeOut(((KnownVariable*) v)->get(), 30));
+					value = fix(CALCULATOR->print(((KnownVariable*) v)->get(), 30));
 				}
 			}
 		} else {
@@ -492,9 +505,15 @@ void print_variable(Variable *v) {
 			}		
 		}
 		if(v->isApproximate()) {
-			value += " (";
-			value += _("approximate");
-			value += ")";
+			if(v == CALCULATOR->v_pi || v == CALCULATOR->v_e || v == CALCULATOR->v_euler || v == CALCULATOR->v_catalan) {
+				value += " (";
+				value += _("variable precision");
+				value += ")";
+			} else {
+				value += " (";
+				value += _("approximate");
+				value += ")";
+			}
 		}
 		fprintf(vfile, "<entry><para>%s</para></entry>\n", value.c_str());
 		fputs("</row>\n", vfile);
@@ -526,10 +545,14 @@ void print_unit(Unit *u) {
 					base_unit += POWER;
 					base_unit += i2s(au->firstBaseExponent());
 				}
-				if(au->firstBaseUnit() == CALCULATOR->u_euro && au->isBuiltin()) {
+				if(au->baseUnit() == CALCULATOR->u_euro && au->isBuiltin()) {
 					relation = "exchange rate";
 				} else {
 					relation = fix(CALCULATOR->localizeExpression(au->expression()).c_str());
+					if(!au->uncertainty().empty()) {
+						relation += "±";
+						relation += fix(CALCULATOR->localizeExpression(au->uncertainty()));
+					}
 				}
 				if(u->isApproximate()) {
 					relation += " (";
@@ -553,7 +576,7 @@ int main(int, char *[]) {
 
 	
 #ifdef ENABLE_NLS
-	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
+	bindtextdomain (GETTEXT_PACKAGE, getPackageLocaleDir().c_str());
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 #endif
